#version 330 core

// Interpolated values from the vertex shaders
in vec2 fragmentUvCoords;
in vec3 worldPosition;
in vec3 cameraNormal;
in vec3 cameraViewDirection;
in vec3 LightDirection_cameraspace;

// Ouput data
out vec3 fragColor;

// Values that stay constant for the whole mesh.
uniform sampler2D colorTexture;
uniform vec3 LightPosition_worldspace;

void main() {

	// Light emission properties
	// You probably want to put them as uniforms
	vec3 LightColor = vec3( 0.988, 0.831, 0.250 ); // daylight sunlight color
	float LightPower = 150.0;
	
	//TODO: Material properties, atm global for all -> change that
	vec3 MaterialDiffuseColor = texture( colorTexture, fragmentUvCoords ).rgb;
	vec3 MaterialAmbientColor = vec3( 0.1, 0.1, 0.1 ) * MaterialDiffuseColor;
	vec3 MaterialSpecularColor = vec3( 0.2, 0.2, 0.2 );

	// Distance to the light
	float distance = length( LightPosition_worldspace - worldPosition );

	// Normal of the computed fragment, in camera space
	vec3 n = normalize( cameraNormal );
	// Direction of the light (from the fragment to the light)
	vec3 l = normalize( LightDirection_cameraspace );
	float cosTheta = clamp( dot( n, l ), 0.0, 1.0 );
	
	// Eye vector (towards the camera)
	vec3 E = normalize( cameraViewDirection );
	// Direction in which the triangle reflects the light
	vec3 R = reflect( -l, n );
	float cosAlpha = clamp( dot( E, R ), 0.0, 1.0 );
	
	fragColor = 
		MaterialAmbientColor + 
		MaterialDiffuseColor * LightColor * LightPower * cosTheta / ( distance * distance ) 
		+ MaterialSpecularColor * LightColor * LightPower * pow( cosAlpha, 5.0 ) / ( distance * distance );
	//float gamma = 1.8;
	//fragColor.rgb = pow( fragColor.rgb, vec3( 1.0 / gamma ) );
}