#version 330 core

layout( location = 0 ) in vec3 vertexPosition;
layout( location = 1 ) in vec3 vertexNormal;
layout( location = 2 ) in vec2 vertexUvCoords;

// Output data ; will be interpolated for each fragment.
out vec2 fragmentUvCoords;
out vec3 worldPosition;
out vec3 cameraNormal;
out vec3 cameraViewDirection;
out vec3 LightDirection_cameraspace;

// Values that stay constant for the whole mesh.
uniform mat4 viewMatrix;
uniform mat4 modelMatrix;
uniform vec3 LightPosition_worldspace;
uniform mat4 viewProjectionMatrix;

void main() {

	// Output position of the vertex, in clip space : MVP * position
	gl_Position = viewProjectionMatrix * modelMatrix * vec4( vertexPosition, 1.0 );

	
	// Position of the vertex, in worldspace : M * position
	worldPosition = ( modelMatrix * vec4( vertexPosition, 1.0 ) ).xyz;
	
	// Vector that goes from the vertex to the camera, in camera space.
	vec3 vertexPosition_cameraspace = ( viewMatrix * modelMatrix * vec4( vertexPosition, 1.0 ) ).xyz;
	cameraViewDirection = vec3( 0.0, 0.0, 0.0 ) - vertexPosition_cameraspace;

	// Vector that goes from the vertex to the light, in camera space. M is ommited because it's identity.
	vec3 LightPosition_cameraspace = ( viewMatrix * vec4( LightPosition_worldspace, 1.0 ) ).xyz;
	LightDirection_cameraspace = LightPosition_cameraspace + cameraViewDirection;
	
	// Normal of the the vertex, in camera space
	cameraNormal = ( viewMatrix * modelMatrix * vec4( vertexNormal, 0.0 ) ).xyz; // Only correct if ModelMatrix does not scale the model ! Use its inverse transpose if not.
	
	fragmentUvCoords = vertexUvCoords;
}

