#version 330 core
out vec4 FragColor;

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
    vec4 FragPosLightSpace;
} fs_in;

uniform sampler2D diffuseTexture;
uniform sampler2D shadowMap;
uniform samplerCube skybox;

uniform vec3 lightPos;
uniform vec3 viewPos;
uniform vec3 cameraPos;

uniform bool envMapEnabled = false;
uniform bool celShadingEnabled = true;
uniform bool isWater = false;

float ShadowCalculation( vec4 fragPosLightSpace ) {
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5;
    float closestDepth = texture( shadowMap, projCoords.xy ).r; 
    float currentDepth = projCoords.z;
    vec3 normal = normalize( fs_in.Normal );
    vec3 lightDir = normalize( lightPos - fs_in.FragPos );
    // PCF
    float shadow = 0.0;
	float bias = 0.000 * tan( acos( dot( normal, lightDir ) ) );
	bias = clamp( bias, 0, 0.005 );
    vec2 texelSize = 1.0 / textureSize( shadowMap, 0 );
    for( int x = -1; x <= 1; ++x ) {
        for( int y = -1; y <= 1; ++y ) {
            float pcfDepth = texture( shadowMap, projCoords.xy + vec2( x, y ) * texelSize ).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0 : 0.0;        
        }    
    }
    shadow /= 9.0;
    
    if( projCoords.z > 1.0 ) {
        shadow = 0.0;
    }
    return shadow;
}

void main() {
	float spec = 0.0;
	float shadow = ShadowCalculation( fs_in.FragPosLightSpace );

	vec3 color = texture( diffuseTexture, fs_in.TexCoords ).rgb;
	vec3 normal = normalize( fs_in.Normal );
	vec3 lightColor = vec3( 0.3 );
	vec3 ambient = 0.5 * color;
	vec3 lightDir = normalize( lightPos - fs_in.FragPos );
	float diff = max( dot( lightDir, normal ), 0.0 );
	vec3 diffuse = diff * lightColor;
	vec3 viewDir = normalize( viewPos - fs_in.FragPos );
	vec3 reflectDir = reflect( -lightDir, normal );
	vec3 halfwayDir = normalize( lightDir + viewDir );
	spec = pow( max( dot( normal, halfwayDir ), 0.0 ), 64.0 );
	vec3 specular = spec * lightColor;
	vec3 lighting = ( ambient + ( 1.0 - shadow ) * ( diffuse + specular ) ) * color;
	FragColor = vec4( lighting, 1.0 );

	if ( celShadingEnabled == true ) {
		float cosTheta = clamp( dot( normal, lightDir ), 0.0, 1.0 );

		if( cosTheta < 0.1 ) {
			FragColor = FragColor * 0.1;
		} else if ( cosTheta < 0.3 ) {
			FragColor = FragColor * 0.3;
		} else if ( cosTheta < 0.6 ) {
			FragColor = FragColor * 0.6;
		}
	}

	if ( envMapEnabled == true ) {
		vec3 I = normalize( fs_in.FragPos - cameraPos );
		vec3 R = reflect( I, normalize( fs_in.Normal ) );
		if ( isWater == true ) {
			FragColor = mix( vec4( texture( skybox, R ).rgb, 1.0 ), FragColor, 0.75 );
		} else {
			FragColor = vec4( texture( skybox, R ).rgb, 1.0 );
		}
	}
}